# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2003-2007  Donald N. Allingham
# Copyright (C) 2008-2010  Brian G. Matherly
# Copyright (C) 2007-2010  Jerome Rapinat
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# -------------------------------------------------------------------------
"""
Catalan-specific classes for relationships.
By Joan Creus
Based on rel_fr.py
Relationship names taken from:
http://www.scgenealogia.org/pdf/Denominacions%20dels%20Parentius.pdf
The only invented name is "cosinastre". Also, "besnetastre" and the like are
not explicitly in the dictionary, but "netastre" is.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------

from gramps.gen.lib import Person
import gramps.gen.relationship

# -------------------------------------------------------------------------

# level is used for generation level:
# at %th generation

_LEVEL_NAME = [
    "",
    "primer",
    "segon",
    "tercer",
    "quart",
    "cinquè",
    "sisè",
    "setè",
    "vuitè",
    "novè",
    "desè",
    "onzè",
    "dotzè",
    "tretzè",
    "catorzè",
    "quinzè",
    "setzè",
    "dissetè",
    "divuitè",
    "dinovè",
    "vintè",
    "vint-i-unè",
    "vint-i-dosè",
    "vint-i-tresè",
    "vint-i-quatrè",
    "vint-i-cinquè",
    "vint-i-sisè",
    "vint-i-setè",
    "vint-i-vuitè",
    "vint-i-novè",
    "trentè",
]

_LEVEL_NAME_F = [
    "",
    "primera",
    "segona",
    "tercera",
    "quarta",
    "cinquena",
    "sisena",
    "setena",
    "vuitena",
    "novena",
    "desena",
    "onzena",
    "dotzena",
    "tretzena",
    "catorzena",
    "quinzena",
    "setzena",
    "dissetena",
    "divuitena",
    "dinovena",
    "vintena",
    "vint-i-unena",
    "vint-i-dosena",
    "vint-i-tresena",
    "vint-i-quatrena",
    "vint-i-cinquena",
    "vint-i-sisena",
    "vint-i-setena",
    "vint-i-vuitena",
    "vint-i-novena",
    "trentena",
]

_LEVEL_NAME_P = [
    "",
    "",  # si són els primers no es posa mai
    "segons",
    "tercers",
    "quarts",
    "cinquens",
    "sisens",
    "setens",
    "vuitens",
    "novens",
    "desens",
    "onzens",
    "dotzens",
    "tretzens",
    "catorzens",
    "quinzens",
    "setzens",
    "dissetens",
    "divuitens",
    "dinovens",
    "vintens",
    "vint-i-unens",
    "vint-i-dosens",
    "vint-i-tresens",
    "vint-i-quatrens",
    "vint-i-cinquens",
    "vint-i-sisens",
    "vint-i-setens",
    "vint-i-vuitens",
    "vint-i-novens",
    "trentens",
]

# small lists, use generation level if > [5]
# the %s is for the inlaw string

_FATHER_LEVEL = [
    "",
    "el pare%s",
    "l'avi%s",
    "el besavi%s",
    "el rebesavi%s",
    "el quadravi%s",
]

_MOTHER_LEVEL = [
    "",
    "la mare%s",
    "l'àvia%s",
    "la besàvia%s",
    "la rebesàvia%s",
    "la quadràvia%s",
]

_MOTHER_LEVEL_STP = [
    "",
    "la madrastra%s",
    "l'aviastra%s",
    "la besaviastra%s",
    "la rebesaviastra%s",
    "la quadraviastra%s",
]

_FATHER_LEVEL_UNK = [
    "",
    "un dels pares%s",
    "un dels avis%s",
    "un dels besavis%s",
    "un dels rebesavis%s",
    "un dels quadravis%s",
]

_FATHER_LEVEL_STP = [
    "",
    "el padrastre%s",
    "l'aviastre%s",
    "el besaviastre%s",
    "el rebesaviastre%s",
    "el quadraviastre%s",
]

_FATHER_LEVEL_STPUNK = [
    "",
    "un dels padrastres%s",
    "un dels aviastres%s",
    "un dels besaviastres%s",
    "un dels rebesaviastres%s",
    "un dels quadraviastres%s",
]

_SON_LEVEL = [
    "",
    "el fill%s",
    "el nét%s",
    "el besnét%s",
    "el rebesnét%s",
    "el quadrinét%s",
]

_DAUGHTER_LEVEL = [
    "",
    "la filla%s",
    "la néta%s",
    "la besnéta%s",
    "la rebesnéta%s",
    "la quadrinéta%s",
]

_SON_LEVEL_UNK = [
    "",
    "un dels fills%s",
    "un dels néts%s",
    "un dels besnéts%s",
    "un dels rebesnéts%s",
    "un dels quadrinéts%s",
]

_SON_LEVEL_STP = [
    "",
    "el fillastre%s",
    "el netastre%s",
    "el besnetastre%s",
    "el rebesnetastre%s",
    "el quadrinetastre%s",
]

_SON_LEVEL_STPUNK = [
    "",
    "un dels fillastres%s",
    "un dels netastres%s",
    "un dels besnetastres%s",
    "un dels rebesnetastres%s",
    "un dels quadrinetastres%s",
]

_DAUGHTER_LEVEL_STP = [
    "",
    "la fillastra%s",
    "la netastra%s",
    "la besnetastra%s",
    "la rebesnetastra%s",
    "la quadrinetastra%s",
]

_BROTHER_LEVEL = [
    "",
    "el germà%s",
    "l'oncle%s",
    "el besoncle%s",
    "el rebesoncle%s",
    "el quadrioncle%s",
]

_SISTER_LEVEL = [
    "",
    "la germana%s",
    "la tia%s",
    "la bestia%s",
    "la rebestia%s",
    "la quadritia%s",
]

_BROTHER_LEVEL_UNK = [
    "",
    "un dels germans%s",
    "un dels oncles%s",
    "un dels besoncles%s",
    "un dels rebesoncles%s",
    "un dels quadrioncles%s",
]

_BROTHER_LEVEL_STP = [
    "",
    "el germanastre%s",
    "l'onclastre%s",
    "el besonclastre%s",
    "el rebesonclastre%s",
    "el quadrionclastre%s",
]

_BROTHER_LEVEL_STPUNK = [
    "",
    "un dels germanastres%s",
    "un dels onclastres%s",
    "un dels besonclastres%s",
    "un dels rebesonclastres%s",
    "un dels quadrionclastres%s",
]

_SISTER_LEVEL_STP = [
    "",
    "la germanastra%s",
    "la tiastra%s",
    "la bestiastra%s",
    "la rebestiastra%s",
    "la quadritiastra%s",
]

_NEPHEW_LEVEL = [
    "",
    "el nebot%s",
    "el besnebot%s",
    "el rebesnebot%s",
    "el quadrinebot%s",
]

_NIECE_LEVEL = [
    "",
    "la neboda%s",
    "la besneboda%s",
    "la rebesneboda%s",
    "la quadrineboda%s",
]

_NEPHEW_LEVEL_UNK = [
    "",
    "un dels nebots%s",
    "un dels besnebots%s",
    "un dels rebesnebots%s",
    "un dels quadrinebots%s",
]

_NEPHEW_LEVEL_STP = [
    "",
    "el nebodastre%s",
    "el besnebodastre%s",
    "el rebesnebodastre%s",
    "el quadrinebodastre%s",
]

_NEPHEW_LEVEL_STPUNK = [
    "",
    "un dels nebodastres%s",
    "un dels besnebodastres%s",
    "un dels rebesnebodastres%s",
    "un dels quadrinebodastres%s",
]

_NIECE_LEVEL_STP = [
    "",
    "la nebodastra%s",
    "la besnebodastra%s",
    "la rebesnebodastra%s",
    "la quadrinebodastra%s",
]

# kinship report

_PARENTS_LEVEL = [
    "",
    "Els pares",
    "Els avis",
    "Els besavis",
    "Els rebesavis",
    "Els quadravis",
]

_CHILDREN_LEVEL = [
    "",
    "Els fills",
    "Els néts",
    "Els besnéts",
    "Els rebesnéts",
    "Els quadrinéts",
]

_SIBLINGS_LEVEL = [
    "",
    "Els germans i les germanes",
    "Els oncles i les ties",
    "Els besoncles i les besties",
    "Els rebesoncles i les rebesties",
    "Els quadrioncles i les quadrities",
]

_NEPHEWS_NIECES_LEVEL = [
    "",
    "Els nebots i les nebodes",
    "Els besnebots i les besnebodes",
    "Els rebesnebots i les rebesnebodes",
    "Els quadrinebots i les quadrinebodes",
]

# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------

# from active person to common ancestor Ga=[level]


def _get_cosi(level, inlaw="", step=""):
    """
    cousins = same level, gender = male
    """
    if step == "":
        nom = "cosí"
    else:
        nom = "cosinastre"

    if level == 1:
        return "el %s germà%s" % (nom, inlaw)
    elif level < len(_LEVEL_NAME):
        return "el %s %s%s" % (nom, _LEVEL_NAME[level], inlaw)
    else:
        return "el %s %dè%s" % (nom, level, inlaw)


def _get_cosina(level, inlaw="", step=""):
    """
    cousins = same level, gender = female
    """
    if inlaw != "":  # polític -> política
        inlaw += "a"
    if step == "":
        nom = "cosina"
    else:
        nom = "cosinastra"

    if level == 1:
        return "la %s germana%s" % (nom, inlaw)
    elif level < len(_LEVEL_NAME):
        return "la %s %s%s" % (nom, _LEVEL_NAME_F[level], inlaw)
    else:
        return "la %s %dena%s" % (nom, level, inlaw)


def _get_cosi_unknown(level, inlaw="", step=""):
    """
    cousins = same level, gender = unknown
    """
    if inlaw != "":  # polític -> polítics
        inlaw += "s"
    if step == "":
        nom = "cosins"
    else:
        nom = "cosinastres"

    if level == 1:
        return "un dels %s germans%s" % (nom, inlaw)
    elif level < len(_LEVEL_NAME):
        return "un dels %s %s%s" % (nom, _LEVEL_NAME_P[level], inlaw)
    else:
        return "un dels %s %dens%s" % (nom, level, inlaw)


def _get_father(level, inlaw="", step=""):
    """
    ancestor, gender = male
    """
    if step == "":
        taula = _FATHER_LEVEL
        nom = "avi"
    else:
        taula = _FATHER_LEVEL_STP
        nom = "aviastre"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME):
        return "l'%s %s%s" % (nom, _LEVEL_NAME[level - 1], inlaw)
    else:
        # use numerical generation

        return "l'%s %dè%s" % (nom, (level - 1), inlaw)


def _get_mother(level, inlaw="", step=""):
    """
    ancestor, gender = female
    """
    if inlaw != "":
        inlaw += "a"
    if step == "":
        taula = _MOTHER_LEVEL
        nom = "àvia"
    else:
        taula = _MOTHER_LEVEL_STP
        nom = "aviastra"

    if level < len(taula):
        return taula[level] % inlaw

        # limitation gen = 30

    elif level <= len(_LEVEL_NAME_F):
        return "l'%s %s%s" % (nom, _LEVEL_NAME_F[level - 1], inlaw)
    else:
        # use numerical generation

        return "l'àvia %dena%s" % ((level - 1), inlaw)


def _get_parent_unknown(level, inlaw="", step=""):
    """
    unknown parent
    """
    if inlaw != "":
        inlaw += "s"

    if step == "":
        taula = _FATHER_LEVEL_UNK
        nom = "avis"
    else:
        taula = _FATHER_LEVEL_STPUNK
        nom = "aviastres"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME_P):
        return "un dels %s %s%s" % (nom, _LEVEL_NAME_P[level - 1], inlaw)
    else:
        # use numerical generation
        return "un dels %s %dens%s" % (nom, (level - 1), inlaw)


def _get_son(level, inlaw="", step=""):
    """
    descendant, gender = male
    """
    if step == "":
        taula = _SON_LEVEL
        nom = "nét"
    else:
        taula = _SON_LEVEL_STP
        nom = "netastre"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME):
        return "el %s %s%s" % (nom, _LEVEL_NAME[level - 1], inlaw)
    else:
        # use numerical generation

        return "el %s %dè%s" % (nom, (level - 1), inlaw)


def _get_sons(level, inlaw=""):
    """
    descendants for kinship report
    """
    if inlaw != "":  # polític -> polítics
        inlaw += "s"

    if inlaw != "" and level == 1:
        return "els gendres i les joves"
    elif level < len(_CHILDREN_LEVEL):
        return _CHILDREN_LEVEL[level] + inlaw
    elif level < len(_LEVEL_NAME_P) - 1:
        return "els néts" + _LEVEL_NAME_P[level - 1] + inlaw
    else:
        return "els néts %dens" % (level - 1) + inlaw


def _get_daughter(level, inlaw="", step=""):
    """
    descendant, gender = female
    """
    if step == "":
        taula = _DAUGHTER_LEVEL
        nom = "néta"
    else:
        taula = _DAUGHTER_LEVEL_STP
        nom = "netastra"

    if inlaw != "":
        inlaw += "a"
    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME_F):
        return "la %s %s%s" % (nom, _LEVEL_NAME_F[level - 1], inlaw)
    else:
        # use numerical generation

        return "la %s %dena%s" % (nom, (level - 1), inlaw)


def _get_child_unknown(level, inlaw="", step=""):
    """
    descendant, gender = unknown
    """
    if inlaw != "":
        inlaw += "s"
    if step == "":
        taula = _SON_LEVEL_UNK
        nom = "néts"
    else:
        taula = _SON_LEVEL_STPUNK
        nom = "netastres"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME_P):
        return "un dels %s %s%s" % (nom, _LEVEL_NAME_P[level - 1], inlaw)
    else:
        # use numerical generation

        return "un dels %s %dens%s" % (nom, (level - 1), inlaw)


def _get_sibling_unknown(level, inlaw="", step=""):
    """
    sibling of an ancestor, gender = unknown
    """
    if inlaw != "":
        inlaw += "s"
    if step == "":
        taula = _BROTHER_LEVEL_UNK
        nom = "oncles"
    else:
        taula = _BROTHER_LEVEL_STPUNK
        nom = "onclastres"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME_P):
        # limitation gen = 30

        return "un dels %s %s%s" % (nom, _LEVEL_NAME_P[level - 1], inlaw)
    else:
        # use numerical generation

        return "un dels %s %dens%s" % (nom, (level - 1), inlaw)


def _get_uncle(level, inlaw="", step=""):
    """
    sibling of an ancestor, gender = male
    """
    if step == "":
        taula = _BROTHER_LEVEL
        nom = "oncle"
    else:
        taula = _BROTHER_LEVEL_STP
        nom = "onclastre"

    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME):
        # limitation gen = 30

        return "l'%s %s%s" % (nom, _LEVEL_NAME[level - 1], inlaw)
    else:
        # use numerical generation

        return "l'%s %dè%s" % (nom, (level - 1), inlaw)


def _get_aunt(level, inlaw="", step=""):
    """
    sibling of an ancestor, gender = female
    """
    if step == "":
        taula = _SISTER_LEVEL
        nom = "tia"
    else:
        taula = _SISTER_LEVEL_STP
        nom = "tiastra"

    if inlaw != "":
        inlaw += "a"
    if level < len(taula):
        return taula[level] % inlaw

    elif level <= len(_LEVEL_NAME_F):
        return "la %s %s%s" % (nom, _LEVEL_NAME_F[level - 1], inlaw)
    else:
        # use numerical generation

        return "la %s %dena%s" % (nom, (level - 1), inlaw)


def _get_uncles(level, inlaw=""):
    """
    siblings for kinship report
    """
    if inlaw != "":
        inlaw += "s"

    if inlaw != "" and level == 1:
        return "els cunyats i les cunyades"
    elif level < len(_SIBLINGS_LEVEL):
        return "%s%s" % (_SIBLINGS_LEVEL[level], inlaw)
    elif level <= len(_LEVEL_NAME_P):
        return "els oncles i les ties %s%s" % (_LEVEL_NAME_P[level - 1], inlaw)
    else:
        # use numerical generation
        return "els oncles i les ties %dens%s" % ((level - 1), inlaw)


def _get_cosins(level, inlaw=""):
    """
    same generation level for kinship report
    """
    if inlaw != "":
        inlaw += "s"

    if level == 2:
        rel_str = "els cosins germans" + inlaw
    elif level <= len(_LEVEL_NAME_P):
        rel_str = "els cosins %s%s" % (_LEVEL_NAME_P[level - 1], inlaw)
    else:
        # security
        rel_str = "els cosins %dens%s" % ((level - 1), inlaw)

    return rel_str


def _get_nephew(level, inlaw="", step=""):
    """
    cousin of a descendant, gender = male
    """
    if step == "":
        taula = _NEPHEW_LEVEL
        nom = "nebot"
    else:
        taula = _NEPHEW_LEVEL_STP
        nom = "nebodastre"

    if level < len(taula):
        return taula[level] % inlaw

    elif level < len(_LEVEL_NAME):
        return "el %s %s%s" % (nom, _LEVEL_NAME[level], inlaw)
    else:
        # use numerical generation

        return "el %s %dè%s" % (nom, level, inlaw)


def _get_nephew_unk(level, inlaw="", step=""):
    """
    cousin of a descendant, gender = unknown
    """
    if inlaw != "":
        inlaw += "s"
    if step == "":
        taula = _NEPHEW_LEVEL_UNK
        nom = "nebots"
    else:
        taula = _NEPHEW_LEVEL_STPUNK
        nom = "nebodastres"

    if level < len(taula):
        return taula[level] % inlaw

    elif level < len(_LEVEL_NAME):
        return "un dels %s %s%s" % (nom, _LEVEL_NAME_P[level], inlaw)
    else:
        # use numerical generation

        return "un dels %s %dens%s" % (nom, level, inlaw)


def _get_niece(level, inlaw="", step=""):
    """
    cousin of a descendant, gender = female
    """
    if inlaw != "":
        inlaw += "a"
    if step == "":
        taula = _NIECE_LEVEL
        nom = "neboda"
    else:
        taula = _NIECE_LEVEL_STP
        nom = "nebodastra"

    if level < len(taula):
        return taula[level] % inlaw

    elif level < len(_LEVEL_NAME_F):
        return "la %s %s%s" % (nom, _LEVEL_NAME_F[level], inlaw)
    else:
        # use numerical generation

        return "la %s %dena%s" % (nom, level, inlaw)


def _get_nephews(level, inlaw=""):
    """
    cousin of a descendant, gender = male
    """
    if inlaw != "":
        inlaw += "s"
    if level <= len(_NEPHEWS_NIECES_LEVEL):
        # limitation gen = 30

        return "%s%s" % (_NEPHEWS_NIECES_LEVEL[level - 1], inlaw)
    elif level <= len(_LEVEL_NAME_P):
        return "els nebots i les nebodes %s%s" % (_LEVEL_NAME_P[level - 1], inlaw)
    else:
        # use numerical generation

        return "els nebots i les nebodes %dens%s" % ((level - 1), inlaw)


def _get_oncle_valencia(levela, levelb, inlaw="", step=""):
    """
    removed cousins, older generations
    """
    if levela <= levelb:
        return "error a _get_oncle_valencia"
    val_level = levela - levelb
    amplada = levelb - 1
    retorn = _get_uncle(val_level + 1, "", step)
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME[amplada]
    return retorn + " valencià " + stramplada + inlaw


def _get_oncles_valencians(levela, levelb, inlaw=""):
    """
    removed cousins, older generations for kinship report
    """
    if inlaw != "":
        inlaw += "s"
    if levela <= levelb:
        return "error a _get_oncles_valencians"
    val_level = levela - levelb
    amplada = levelb - 1
    retorn = _get_uncles(val_level + 1, "")
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME_P[amplada]
    return retorn + " valencians " + stramplada + inlaw


def _get_nebot_valencia(levela, levelb, inlaw="", step=""):
    """
    removed cousins, younger generations
    """
    if levelb <= levela:
        return "error a _get_nebot_valencia"
    val_level = levelb - levela
    amplada = levela - 1
    retorn = _get_nephew(val_level, "", step)
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME[amplada]
    return retorn + " valencià " + stramplada + inlaw


def _get_nebots_valencians(levela, levelb, inlaw=""):
    """
    removed cousins, younger generations, for kinship report
    gender = male
    """
    if inlaw != "":
        inlaw += "s"
    if levelb <= levela:
        return "error a _get_nebots_valencians"
    val_level = levelb - levela
    amplada = levela - 1
    retorn = _get_nephews(val_level + 1, "")
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME_P[amplada]
    return retorn + " valencians " + stramplada + inlaw


def _get_tia_valenciana(levela, levelb, inlaw="", step=""):
    """
    removed cousins, older generations
    gender = female
    """
    if inlaw != "":
        inlaw += "a"
    if levela <= levelb:
        return "error a _get_tia_valenciana"
    val_level = levela - levelb
    amplada = levelb - 1
    retorn = _get_aunt(val_level + 1, "", step)
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME_F[amplada]
    return retorn + " valenciana " + stramplada + inlaw


def _get_neboda_valenciana(levela, levelb, inlaw="", step=""):
    """
    removed cousins, younger generations
    gender = female
    """
    if inlaw != "":
        inlaw += "a"
    if levelb <= levela:
        return "error a _get_neboda_valenciana"
    val_level = levelb - levela
    amplada = levela - 1
    retorn = _get_niece(val_level, "", step)
    if amplada == 1:
        stramplada = ""
    else:
        stramplada = _LEVEL_NAME_F[amplada]
    return retorn + " valenciana " + stramplada + inlaw


class RelationshipCalculator(gramps.gen.relationship.RelationshipCalculator):
    """
    RelationshipCalculator Class
    """

    INLAW = " polític"

    def __init__(self):
        gramps.gen.relationship.RelationshipCalculator.__init__(self)

    # kinship report

    def get_plural_relationship_string(
        self,
        Ga,
        Gb,
        reltocommon_a="",
        reltocommon_b="",
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """
        see relationship.py
        """

        rel_str = "parents llunyans"
        # atgen = " de la %sena generació"
        # bygen = " per la %sena generació"
        # cmt = " (germans o germanes d'un avantpassat" + atgen % Ga + ")"

        if in_law_a or in_law_b:
            inlaw = self.INLAW
        else:
            inlaw = ""

        if Ga == 0:
            # These are descendants

            rel_str = _get_sons(Gb, inlaw)

        elif Gb == 0:
            # These are parents/grand parents

            if Ga < len(_PARENTS_LEVEL):
                rel_str = _PARENTS_LEVEL[Ga]
            elif Ga < len(_LEVEL_NAME_P) - 1:
                rel_str = "els avis " + _LEVEL_NAME_P[Ga - 1]
            else:
                rel_str = "els avis %dens" % (Ga - 1)

        elif Gb == 1:
            # These are siblings/aunts/uncles
            rel_str = _get_uncles(Ga, inlaw)

        elif Ga == 1:
            # These are nieces/nephews

            rel_str = _get_nephews(Gb, inlaw)

        elif Ga > 1:
            if Ga == Gb:
                # These are cousins in the same generation

                rel_str = _get_cosins(Ga, inlaw)
            elif Ga > Gb:
                # These are cousins in different generations with the second
                # person being in a higher generation from the common ancestor
                # than the first person.

                rel_str = _get_oncles_valencians(Ga, Gb, inlaw)

            elif Gb > Ga:
                # These are cousins in different generations with the second
                # person being in a higher generation from the common ancestor
                # than the first person.

                rel_str = _get_nebots_valencians(Ga, Gb, inlaw)

        else:
            rel_str = "Error in get_plural_relationship_string()"

        return rel_str

    # quick report (missing on RelCalc tool - Status Bar)

    def get_single_relationship_string(
        self,
        Ga,
        Gb,
        gender_a,
        gender_b,
        reltocommon_a,
        reltocommon_b,
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """
        see relationship.py
        """

        if only_birth:
            step = ""
        else:
            step = self.STEP

        if in_law_a or in_law_b:
            inlaw = self.INLAW
        else:
            inlaw = ""

        rel_str = "un parent llunyà%s" % inlaw
        # bygen = " per la %sena generació"
        if Ga == 0:
            # b is descendant of a

            if Gb == 0:
                rel_str = "la mateixa persona"
            elif gender_b == Person.MALE:
                # spouse of daughter

                if inlaw and Gb == 1 and not step:
                    rel_str = "el gendre"
                else:
                    rel_str = _get_son(Gb, inlaw, step)
            elif gender_b == Person.FEMALE:
                # spouse of son

                if inlaw and Gb == 1 and not step:
                    rel_str = "la jove"
                else:
                    rel_str = _get_daughter(Gb, inlaw, step)
            else:
                rel_str = _get_child_unknown(Gb, inlaw, step)
        elif Gb == 0:
            # b is parents/grand parent of a

            if gender_b == Person.MALE:
                # father of spouse (family of spouse)
                if Ga == 1 and inlaw and not step:
                    rel_str = "el sogre"

                else:
                    rel_str = _get_father(Ga, inlaw, step)
            elif gender_b == Person.FEMALE:
                # mother of spouse (family of spouse)
                if Ga == 1 and inlaw and not step:
                    rel_str = "la sogra"

                else:
                    rel_str = _get_mother(Ga, inlaw, step)
            else:
                rel_str = _get_parent_unknown(Ga, inlaw, step)
        elif Gb == 1:
            # b is sibling/aunt/uncle of a

            if inlaw and Ga == 1 and not step:
                if gender_b == Person.MALE:
                    rel_str = "el cunyat"
                elif gender_b == Person.FEMALE:
                    rel_str = "la cunyada"
                else:  # gender_b == Person.UNKNOWN:
                    rel_str = "un cunyat"

            elif gender_b == Person.MALE:
                rel_str = _get_uncle(Ga, inlaw, step)
            elif gender_b == Person.FEMALE:
                rel_str = _get_aunt(Ga, inlaw, step)
            else:
                rel_str = _get_sibling_unknown(Ga, inlaw, step)

        elif Ga == 1:
            # b is niece/nephew of a

            if gender_b == Person.MALE:
                rel_str = _get_nephew(Gb - 1, inlaw, step)
            elif gender_b == Person.FEMALE:
                rel_str = _get_niece(Gb - 1, inlaw, step)
            elif gender_b == Person.UNKNOWN:
                rel_str = _get_nephew_unk(Gb - 1, inlaw, step)
            else:
                # This should never get executed
                rel_str = "un nebot%s llunyà (%dena generació)" % (inlaw, Gb)
        elif Ga == Gb:
            # a and b cousins in the same generation

            if gender_b == Person.MALE:
                rel_str = _get_cosi(Ga - 1, inlaw, step)
            elif gender_b == Person.FEMALE:
                rel_str = _get_cosina(Ga - 1, inlaw, step)
            elif gender_b == Person.UNKNOWN:
                rel_str = _get_cosi_unknown(Ga - 1, inlaw, step)
            else:
                rel_str = "error in get_single_relationship_string()"
        elif Ga > 1 and Ga > Gb:
            # These are cousins in different generations with the second person
            # being in a higher generation from the common ancestor than the
            # first person.

            if gender_b == Person.MALE:
                rel_str = _get_oncle_valencia(Ga, Gb, inlaw, step)
            elif gender_b == Person.FEMALE:
                rel_str = _get_tia_valenciana(Ga, Gb, inlaw, step)
            else:
                rel_str = _get_oncle_valencia(Ga, Gb, inlaw, step)
        elif Gb > 1 and Gb > Ga:
            # These are cousins in different generations with the second person
            # being in a lower generation from the common ancestor than the
            # first person.
            if gender_b == Person.MALE:
                rel_str = _get_nebot_valencia(Ga, Gb, inlaw, step)
            elif gender_b == Person.FEMALE:
                rel_str = _get_neboda_valenciana(Ga, Gb, inlaw, step)
            else:
                rel_str = _get_nebot_valencia(Ga, Gb, inlaw, step)

        return rel_str

    # RelCalc tool - Status Bar

    def get_sibling_relationship_string(
        self, sib_type, gender_a, gender_b, in_law_a=False, in_law_b=False
    ):
        """
        voir relationship.py
        """

        if in_law_a or in_law_b:
            inlaw = self.INLAW
        else:
            inlaw = ""

        if sib_type == self.NORM_SIB:
            if not inlaw:
                if gender_b == Person.MALE:
                    rel_str = "el germà"
                elif gender_b == Person.FEMALE:
                    rel_str = "la germana"
                else:
                    rel_str = "el germà o germana"
            else:
                if gender_b == Person.MALE:
                    rel_str = "el cunyat"
                elif gender_b == Person.FEMALE:
                    rel_str = "la cunyada"
                else:
                    rel_str = "el cunyat o la cunyada"
        elif sib_type == self.UNKNOWN_SIB:
            if not inlaw:
                if gender_b == Person.MALE:
                    rel_str = "el germà"
                elif gender_b == Person.FEMALE:
                    rel_str = "la germana"
                else:
                    rel_str = "el germà o germana"
            else:
                if gender_b == Person.MALE:
                    rel_str = "el cunyat"
                elif gender_b == Person.FEMALE:
                    rel_str = "la cunyada"
                else:
                    rel_str = "el cunyat o la cunyada"
        elif (
            sib_type == self.HALF_SIB_MOTHER
            or sib_type == self.HALF_SIB_FATHER
            or sib_type == self.STEP_SIB
        ):
            if not inlaw:
                if gender_b == Person.MALE:
                    rel_str = "el germanastre"
                elif gender_b == Person.FEMALE:
                    rel_str = "la germanastra"
                else:
                    rel_str = "el germanastre o la germanastra"
            else:
                if gender_b == Person.MALE:
                    rel_str = "el cunyat"
                elif gender_b == Person.FEMALE:
                    rel_str = "la cunyada"
                else:
                    rel_str = "el cunyat o la cunyada"
        return rel_str


if __name__ == "__main__":
    # Test function. Call it as follows from the command line (so as to find
    #        imported modules):
    #    export PYTHONPATH=/path/to/gramps/src
    # python src/plugins/rel/rel_ca.py
    # (Above not needed here)

    # ""
    #   TRANSLATORS, copy this if statement at the bottom of your
    #   rel_xx.py module, and test your work with:
    #   python src/plugins/rel/rel_xx.py
    # ""
    from gramps.gen.relationship import test

    RC = RelationshipCalculator()
    test(RC, True)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Python classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from .embeddedlist import TEXT_COL, MARKUP_COL, ICON_COL
from .ldsembedlist import LdsEmbedList
from gramps.gen.lib import LdsOrd


# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------
class FamilyLdsEmbedList(LdsEmbedList):
    _HANDLE_COL = 7
    #    _DND_TYPE = DdTargets.ADDRESS

    # index = column in model. Value =
    #  (name, sortcol in model, width, markup/text, weigth_col
    _column_names = [
        (_("Type"), 0, 150, TEXT_COL, -1, None),
        (_("Date"), 1, 150, MARKUP_COL, -1, None),
        (_("Status"), 3, 75, TEXT_COL, -1, None),
        (_("Temple"), 2, 200, TEXT_COL, -1, None),
        (_("Place"), 4, 100, TEXT_COL, -1, None),
        (_("Source"), 5, 30, ICON_COL, -1, "gramps-source"),
        (_("Private"), 6, 30, ICON_COL, -1, "gramps-lock"),
    ]

    def __init__(self, dbstate, uistate, track, data, config_key):
        LdsEmbedList.__init__(self, dbstate, uistate, track, data, config_key)

    def get_editor(self):
        from .. import EditFamilyLdsOrd

        return EditFamilyLdsOrd

    def new_data(self):
        lds = LdsOrd()
        lds.set_type(LdsOrd.SEAL_TO_SPOUSE)
        return lds



"""
Use with:
python3 setup.py build_exe --no-compress -O1
"""

import sys
import os
import site
import subprocess
import tempfile
import atexit
import shutil
import zipfile
import cx_Freeze

sys.path.insert(0, "dist")
import gramps
from gramps.version import VERSION_TUPLE

try:
    from gramps.version import VERSION_QUALIFIER
except:
    VERSION_QUALIFIER = ""

# import logging
# logging.basicConfig(level=logging.DEBUG)
VQ = {
    "-alpha1": 10,
    "-alpha2": 11,
    "-alpha3": 12,
    "-beta1": 21,
    "-beta2": 22,
    "-beta3": 23,
    "-rc1": 22,
    "": 0,
}


def normalize(version):
    version = version.replace("-", "")
    version = version.replace("alpha", "a")
    version = version.replace("beta", "b")
    return version


FULL_VERSION = ".".join(map(str, VERSION_TUPLE)) + VERSION_QUALIFIER
NORMALIZED_VERSION = normalize(FULL_VERSION)

VERSION = ".".join(map(str, VERSION_TUPLE)) + "." + str(VQ.get(VERSION_QUALIFIER, 99))
COPYRIGHT = "Copyright 2024, Gramps developers.  GNU General Public License"
BASE_DIR = os.path.split(sys.base_prefix)[1]
SETUP_DIR = os.path.dirname(os.path.realpath(__file__))
SETUP_FILES = [
    "setup.py",
    "gramps.ico",
    "grampsc.ico",
    "grampsd.ico",
    "grampsaioc.py",
    "grampsaiocd.py",
    "grampsaiow.py",
]
if "32" in BASE_DIR:
    SETUP_FILES.append("".join(("grampsaio", "32", ".nsi")))
else:
    SETUP_FILES.append("".join(("grampsaio", "64", ".nsi")))

INCLUDE_DLL_PATH = os.path.join(sys.base_exec_prefix, "bin")
INCLUDE_FILES = []
INCLUDES = ["gi", "cgi", "colorsys", "site"]
PACKAGES = [
    "gi",
    "cairo",
    "xml",
    "bsddb3",
    "lxml",
    "PIL",
    "json",
    "csv",
    "sqlite3",
    "cProfile",
    "networkx",
    "psycopg2",
    "requests",
    "logging",
    "html",
    "compileall",
    "graphviz",
    "pydotplus",
    "pygraphviz",
    "pydot",
    "orjson",
    "selenium",
]
EXCLUDES = [
    "tkinter",
    "PyQt5",
    "PyQt5.QtCore",
    "PyQt5.QtGui",
    "pyside" "PyQt5.QtWidgets",
    "sip",
    "PIL.ImageQt",
    "pip",
    "distlib",
]

REPLACE_PATHS = [
    ("*", "AIO/"),
]
MISSING_DLL = [
    "libgtk-3-0.dll",
    "libgspell-1-3.dll",
    "libgexiv2-2.dll",
    "libgoocanvas-3.0-9.dll",
    "libosmgpsmap-1.0-1.dll",
    "gswin32c.exe",
    "dot.exe",
    "libgvplugin_core-6.dll",
    "libgvplugin_dot_layout-6.dll",
    "libgvplugin_gd-6.dll",
    "libgvplugin_pango-6.dll",
    "libgvplugin_rsvg-6.dll",
    "glib-compile-schemas.exe",
    "gdk-pixbuf-query-loaders.exe",
    "gtk-update-icon-cache-3.0.exe",
    "fc-cache.exe",
    "fc-match.exe",
    "gspawn-win64-helper-console.exe",
    "gspawn-win64-helper.exe",
    "libgeocode-glib-0.dll",
    "gdbus.exe",
]
BIN_EXCLUDES = ["Qt5Core.dll", "gdiplus.dll", "gdiplus"]

from os.path import dirname, basename
import pip

libpip_path = dirname(pip.__file__)
pip_path = os.path.join(libpip_path, "__main__.py")
INCLUDE_FILES.append((libpip_path, "lib/pip"))
import distlib

libdistlib_path = dirname(distlib.__file__)
INCLUDE_FILES.append((libdistlib_path, "lib/distlib"))

os.makedirs(os.path.join(BASE_DIR, "var/cache/fontconfig"), exist_ok=True)
for file in SETUP_FILES:
    INCLUDE_FILES.append((os.path.join(SETUP_DIR, file), os.path.join("src", file)))
for dll in MISSING_DLL:
    INCLUDE_FILES.append(
        (os.path.join(INCLUDE_DLL_PATH, dll), os.path.join("lib", dll))
    )
MISSING_LIBS = [
    "lib/enchant-2",
    "lib/gdk-pixbuf-2.0",
    "lib/girepository-1.0",
    "share/enchant",
    "share/glib-2.0/schemas",
    "share/xml/iso-codes",
    "etc/gtk-3.0",
    "etc/ssl/certs",
    "etc/ssl/cert.pem",
    "etc/fonts",
    "lib/gio",
    "share/icons/gnome",
    "share/icons/hicolor",
    "share/icons/gramps.png",
    "share/icons/Adwaita/icon-theme.cache",
    "share/icons/Adwaita/index.theme",
    "share/locale",
]
ADWAITA = [
    "16x16",
    "scalable",
    "symbolic",
]
for adw in ADWAITA:
    INCLUDE_FILES.append(
        (
            os.path.join(sys.base_prefix, "share/icons/Adwaita", adw),
            os.path.join("share/icons/Adwaita", adw),
        )
    )
for lib in MISSING_LIBS:
    INCLUDE_FILES.append((os.path.join(sys.base_prefix, lib), lib))

INCLUDE_FILES.append("dist/gramps")
INCLUDE_FILES.append(
    ("dist/gramps-" + NORMALIZED_VERSION + ".data/data/share", "share")
)
EXECUTABLES = [
    cx_Freeze.Executable(
        "grampsaioc.py",
        base="Console",
        target_name="gramps.exe",
        icon="grampsc.ico",
        copyright=COPYRIGHT,
    ),
    cx_Freeze.Executable(
        "grampsaiow.py",
        base="Win32GUI",
        target_name="grampsw.exe",
        icon="gramps.ico",
        copyright=COPYRIGHT,
    ),
    cx_Freeze.Executable(
        "grampsaiocd.py",
        base="Console",
        target_name="grampsd.exe",
        icon="grampsd.ico",
        copyright=COPYRIGHT,
    ),
    cx_Freeze.Executable(
        pip_path, base="Console", target_name="pip.exe", icon="grampsc.ico"
    ),
]
BUILD_EXE_OPTIONS = {
    "packages": PACKAGES,
    "includes": INCLUDES,
    "excludes": EXCLUDES,
    "include_files": INCLUDE_FILES,
    "bin_includes": MISSING_DLL,
    "zip_include_packages": "*",  # ZIP_INCLUDE_PACKAGES,
    "zip_exclude_packages": EXCLUDES,
    "bin_excludes": BIN_EXCLUDES,
    "replace_paths": REPLACE_PATHS,
    "build_exe": BASE_DIR,
}
BDIST_MSI_OPTIONS = {  # uuid.uuid5(uuid.NAMESPACE_DNS, 'GrampsAIO64-5-trunk')
    "upgrade_code": "{fbccc04b-7b2e-56d3-8bb7-94d5f68de822}",
    # uuid.uuid5(uuid.NAMESPACE_DNS, 'v5.0.0-alpha1-476-g473d3aa')
    "product_code": "{48304362-2945-5a10-ad60-241f233be4d2}",
    "add_to_path": False,
    #'initial_target_dir': r'[ProgramFilesFolder]\%s\%s' %
    # (company_name, product_name),
}
cx_Freeze.setup(
    name="GrampsAIO32" if "32" in BASE_DIR else "GrampsAIO64",
    options={"build_exe": BUILD_EXE_OPTIONS, "bdist_msi": BDIST_MSI_OPTIONS},
    version=VERSION,
    description="Gramps Genealogy software",
    long_description=VERSION_QUALIFIER,
    executables=EXECUTABLES,
)

ZIN = zipfile.ZipFile(os.path.join(BASE_DIR, "lib/library.zip"), "r")
ZOUT = zipfile.ZipFile(os.path.join(BASE_DIR, "lib/pythonx.zip"), "w")
for item in ZIN.infolist():
    if not os.path.dirname(item.filename).startswith("gramps"):
        # if '/test' in item.filename or 'test/' in item.filename:
        #    print("Zip Excluded:", item.filename)
        # else:
        print("Zip Included:", item.filename)
        buffer = ZIN.read(item.filename)
        ZOUT.writestr(item, buffer)
ZOUT.close()
ZIN.close()
shutil.move(
    os.path.join(BASE_DIR, "lib/pythonx.zip"), os.path.join(BASE_DIR, "lib/library.zip")
)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Lithuanian-specific classes for parsing and displaying dates.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import re

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler


# -------------------------------------------------------------------------
#
# Lithuanian parser
#
# -------------------------------------------------------------------------
class DateParserLT(DateParser):
    """
    Convert a text string into a Date object. If the date cannot be
    converted, the text string is assigned.
    """

    month_to_int = DateParser.month_to_int

    # Custom months not the same as long months

    month_to_int["sausis"] = 1
    month_to_int["vasaris"] = 2
    month_to_int["kovas"] = 3
    month_to_int["balandis"] = 4
    month_to_int["gegužė"] = 5
    month_to_int["gegužis"] = 5
    month_to_int["birželis"] = 6
    month_to_int["liepa"] = 7
    month_to_int["rugpjūtis"] = 8
    month_to_int["rugsėjis"] = 9
    month_to_int["spalis"] = 10
    month_to_int["lapkritis"] = 11
    month_to_int["gruodis"] = 12

    # For not full months

    month_to_int["saus"] = 1
    month_to_int["vasa"] = 2
    month_to_int["vasar"] = 2
    month_to_int["bala"] = 4
    month_to_int["balan"] = 4
    month_to_int["baland"] = 4
    month_to_int["gegu"] = 5
    month_to_int["geguž"] = 5
    month_to_int["birž"] = 6
    month_to_int["birže"] = 6
    month_to_int["biržel"] = 6
    month_to_int["liep"] = 7
    month_to_int["rugp"] = 8
    month_to_int["rugpj"] = 8
    month_to_int["rugpjū"] = 8
    month_to_int["rugpjūt"] = 8
    month_to_int["rugs"] = 9
    month_to_int["rugsė"] = 9
    month_to_int["rugsėj"] = 9
    month_to_int["rugsėjis"] = 9
    month_to_int["spal"] = 10
    month_to_int["lapk"] = 11
    month_to_int["lapkr"] = 11
    month_to_int["lapkri"] = 11
    month_to_int["lapkrit"] = 11
    month_to_int["gru"] = 12
    month_to_int["gruo"] = 12
    month_to_int["gruod"] = 12

    modifier_to_int = {
        "prieš": Date.MOD_BEFORE,
        "po": Date.MOD_AFTER,
        "apie": Date.MOD_ABOUT,
        "from": Date.MOD_FROM,
        "to": Date.MOD_TO,
    }

    calendar_to_int = {
        "grigaliaus": Date.CAL_GREGORIAN,
        "g": Date.CAL_GREGORIAN,
        "julijaus": Date.CAL_JULIAN,
        "j": Date.CAL_JULIAN,
        "hebrajų": Date.CAL_HEBREW,
        "h": Date.CAL_HEBREW,
        "islamo": Date.CAL_ISLAMIC,
        "i": Date.CAL_ISLAMIC,
        "prancūzų respublikos": Date.CAL_FRENCH,
        "r": Date.CAL_FRENCH,
        "persų": Date.CAL_PERSIAN,
        "p": Date.CAL_PERSIAN,
        "švedų": Date.CAL_SWEDISH,
        "s": Date.CAL_SWEDISH,
    }

    quality_to_int = {
        "apytikriai": Date.QUAL_ESTIMATED,
        "apskaičiuota": Date.QUAL_CALCULATED,
    }

    def init_strings(self):
        DateParser.init_strings(self)
        # this next RE has the (possibly-slashed) year at the string's start
        self._text2 = re.compile(
            r"((\d+)(/\d+)?)?\s+?m\.\s+%s\s*(\d+)?\s*d?\.?$" % self._mon_str,
            re.IGNORECASE,
        )
        _span_1 = ["nuo"]
        _span_2 = ["iki"]
        _range_1 = ["tarp"]
        _range_2 = ["ir"]
        self._span = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_span_1), "|".join(_span_2)),
            re.IGNORECASE,
        )
        self._range = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_range_1), "|".join(_range_2)),
            re.IGNORECASE,
        )


# -------------------------------------------------------------------------
#
# Lithuanian displayer
#
# -------------------------------------------------------------------------
class DateDisplayLT(DateDisplay):
    """
    Lithuanian language date display class.
    """

    long_months = (
        "",
        "sausio",
        "vasario",
        "kovo",
        "balandžio",
        "gegužės",
        "birželio",
        "liepos",
        "rugpjūčio",
        "rugsėjo",
        "spalio",
        "lapkričio",
        "gruodžio",
    )

    long_months_vardininkas = (
        "",
        "sausis",
        "vasaris",
        "kovas",
        "balandis",
        "gegužė",
        "birželis",
        "liepa",
        "rugpjūtis",
        "rugsėjis",
        "spalis",
        "lapkritis",
        "gruodis",
    )

    short_months = (
        "",
        "Sau",
        "Vas",
        "Kov",
        "Bal",
        "Geg",
        "Bir",
        "Lie",
        "Rgp",
        "Rgs",
        "Spa",
        "Lap",
        "Grd",
    )

    calendar = (
        "",
        "julijaus",
        "hebrajų",
        "prancūzų respublikos",
        "persų",
        "islamo",
        "švedų",
    )

    _mod_str = ("", "prieš ", "po ", "apie ", "", "", "")

    _qual_str = ("", "apytikriai ", "apskaičiuota ")

    formats = (
        "mmmm-MM-DD (ISO)",
        "mmmm.MM.DD",
        "mmmm m. mėnesio diena d.",
        "Mėn diena, metai",
    )
    # this definition must agree with its "_display_gregorian" method

    def _display_gregorian(self, date_val, **kwargs):
        """
        display gregorian calendar date in different format
        """
        # this must agree with its locale-specific "formats" definition
        value = self.display_iso(date_val)
        year = self._slash_year(date_val[2], date_val[3])
        if self.format == 0:
            return self.display_iso(date_val)
        elif self.format == 1:
            # numerical
            return self.dd_dformat01(date_val)
        elif self.format == 2:
            # mmmm m. mėnesio diena d. (year m. month_name day d.)
            if date_val[0] == 0:
                if date_val[1] == 0:
                    value = year
                else:
                    value = "%s m. %s" % (
                        year,
                        self.long_months_vardininkas[date_val[1]],
                    )
            else:
                value = "%s m. %s %d d." % (
                    year,
                    self.long_months[date_val[1]],
                    date_val[0],
                )
        elif self.format == 3:
            # month_abbreviation day, year
            if date_val[0] == 0:
                if date_val[1] == 0:
                    value = year
                else:
                    value = "%s %s" % (self.short_months[date_val[1]], year)
            else:
                value = "%s %d, %s" % (
                    self.short_months[date_val[1]],
                    date_val[0],
                    year,
                )
        if date_val[2] < 0:
            return self._bce_str % value
        else:
            return value

    def display(self, date):
        """
        Return a text string representing the date.
        """
        mod = date.get_modifier()
        cal = date.get_calendar()
        qual = date.get_quality()
        start = date.get_start_date()
        newyear = date.get_new_year()

        qual_str = self._qual_str[qual]

        if mod == Date.MOD_TEXTONLY:
            return date.get_text()
        elif start == Date.EMPTY:
            return ""
        elif mod == Date.MOD_SPAN:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%s%s %s %s %s%s" % (qual_str, "nuo", d1, "iki", d2, scal)
        elif mod == Date.MOD_RANGE:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%s%s %s %s %s%s" % (qual_str, "tarp", d1, "ir", d2, scal)
        else:
            text = self.display_cal[date.get_calendar()](start)
            scal = self.format_extras(cal, newyear)
            return "%s%s%s%s" % (qual_str, self._mod_str[mod], text, scal)


# -------------------------------------------------------------------------
#
# Register classes
#
# -------------------------------------------------------------------------
register_datehandler(
    ("lt_LT", "lt", "lithuanian", "Lithuanian", ("%Y.%m.%d",)),
    DateParserLT,
    DateDisplayLT,
)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2008,2012  Brian G. Matherly
# Copyright (C) 2010       Jakim Friant
# Copyright (C) 2013-2014  Paul Franklin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Reports/Text Reports/Database Summary Report.
"""

# ------------------------------------------------------------------------
#
# standard python modules
#
# ------------------------------------------------------------------------
import os

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
from gramps.gen.lib import Person
from gramps.gen.plug.report import Report
from gramps.gen.plug.report import utils
from gramps.gen.plug.report import MenuReportOptions
from gramps.gen.plug.report import stdoptions
from gramps.gen.plug.docgen import (
    IndexMark,
    FontStyle,
    ParagraphStyle,
    FONT_SANS_SERIF,
    INDEX_TYPE_TOC,
    PARA_ALIGN_CENTER,
)
from gramps.gen.utils.file import media_path_full
from gramps.gen.datehandler import get_date
from gramps.gen.proxy import CacheProxyDb


# ------------------------------------------------------------------------
#
# SummaryReport
#
# ------------------------------------------------------------------------
class SummaryReport(Report):
    """
    This report produces a summary of the objects in the database.
    """

    def __init__(self, database, options, user):
        """
        Create the SummaryReport object that produces the report.

        The arguments are:

        database        - the Gramps database instance
        options         - instance of the Options class for this report
        user            - a gen.user.User() instance
        incl_private    - Whether to count private data
        living_people - How to handle living people
        years_past_death - Consider as living this many years after death
        """
        Report.__init__(self, database, options, user)

        self.set_locale(options.menu.get_option_by_name("trans").get_value())

        stdoptions.run_private_data_option(self, options.menu)
        stdoptions.run_living_people_option(self, options.menu, self._locale)
        self.database = CacheProxyDb(self.database)
        self.__db = self.database

    def write_report(self):
        """
        Overridden function to generate the report.
        """
        self.doc.start_paragraph("SR-Title")
        title = self._("Database Summary Report")
        mark = IndexMark(title, INDEX_TYPE_TOC, 1)
        self.doc.write_text(title, mark)
        self.doc.end_paragraph()

        self.summarize_people()
        self.summarize_families()
        self.summarize_events()
        self.summarize_places()
        self.summarize_sources()
        self.summarize_citations()
        self.summarize_repositories()
        self.summarize_media()
        self.summarize_notes()

    def summarize_people(self):
        """
        Write a summary of all the people in the database.
        """
        with_media = 0
        incomp_names = 0
        disconnected = 0
        missing_bday = 0
        males = 0
        females = 0
        others = 0
        unknowns = 0
        namelist = []

        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Individuals"))
        self.doc.end_paragraph()

        num_people = 0
        for person in self.__db.iter_people():
            num_people += 1
            primary_names = [person.get_primary_name()]

            # Count people with media.
            length = len(person.get_media_list())
            if length > 0:
                with_media += 1

            # Count people with incomplete names.
            for name in primary_names + person.get_alternate_names():
                if name.get_first_name().strip() == "":
                    incomp_names += 1
                else:
                    if name.get_surname_list():
                        for surname in name.get_surname_list():
                            if surname.get_surname().strip() == "":
                                incomp_names += 1
                    else:
                        incomp_names += 1

            # Count people without families.
            if not person.get_main_parents_family_handle() and not len(
                person.get_family_handle_list()
            ):
                disconnected += 1

            # Count missing birthdays.
            birth_ref = person.get_birth_ref()
            if birth_ref:
                birth = self.__db.get_event_from_handle(birth_ref.ref)
                if not get_date(birth):
                    missing_bday += 1
            else:
                missing_bday += 1

            # Count genders.
            if person.get_gender() == Person.FEMALE:
                females += 1
            elif person.get_gender() == Person.MALE:
                males += 1
            elif person.get_gender() == Person.OTHER:
                others += 1
            else:
                unknowns += 1

            # Count unique surnames
            for name in primary_names + person.get_alternate_names():
                if (
                    not name.get_surname().strip() in namelist
                    and not name.get_surname().strip() == ""
                ):
                    namelist.append(name.get_surname().strip())

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Number of individuals: %d") % num_people)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Males: %d") % males)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Females: %d") % females)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Individuals with other gender: %d") % others)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Individuals with unknown gender: %d") % unknowns)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Incomplete names: %d") % incomp_names)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Individuals missing birth dates: %d") % missing_bday
        )
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Disconnected individuals: %d") % disconnected)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Unique surnames: %d") % len(namelist))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Individuals with media objects: %d") % with_media)
        self.doc.end_paragraph()

    def summarize_families(self):
        """
        Write a summary of all the families in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Family Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of families: %d") % self.__db.get_number_of_families()
        )
        self.doc.end_paragraph()

    def summarize_events(self):
        """
        Write a summary of all the events in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Event Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of events: %d") % self.__db.get_number_of_events()
        )
        self.doc.end_paragraph()

    def summarize_places(self):
        """
        Write a summary of all the places in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Place Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of places: %d") % self.__db.get_number_of_places()
        )
        self.doc.end_paragraph()

    def summarize_sources(self):
        """
        Write a summary of all the sources in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Source Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of sources: %d") % self.__db.get_number_of_sources()
        )
        self.doc.end_paragraph()

    def summarize_citations(self):
        """
        Write a summary of all the citations in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Citation Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of citations: %d") % self.__db.get_number_of_citations()
        )
        self.doc.end_paragraph()

    def summarize_repositories(self):
        """
        Write a summary of all the repositories in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Repository Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of repositories: %d")
            % self.__db.get_number_of_repositories()
        )
        self.doc.end_paragraph()

    def summarize_media(self):
        """
        Write a summary of all the media in the database.
        """
        total_media = 0
        size_in_bytes = 0
        notfound = []

        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Media Objects"))
        self.doc.end_paragraph()

        total_media = len(self.__db.get_media_handles())
        mbytes = "0"
        for media_id in self.__db.get_media_handles():
            media = self.__db.get_media_from_handle(media_id)
            try:
                size_in_bytes += os.path.getsize(
                    media_path_full(self.__db, media.get_path())
                )
                length = len(str(size_in_bytes))
                if size_in_bytes <= 999999:
                    mbytes = self._("less than 1")
                else:
                    mbytes = str(size_in_bytes)[: (length - 6)]
            except:
                notfound.append(media.get_path())

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Number of unique media objects: %d") % total_media)
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(self._("Total size of media objects: %s MB") % mbytes)
        self.doc.end_paragraph()

        if len(notfound) > 0:
            self.doc.start_paragraph("SR-Heading")
            self.doc.write_text(self._("Missing Media Objects"))
            self.doc.end_paragraph()

            for media_path in notfound:
                self.doc.start_paragraph("SR-Normal")
                self.doc.write_text(media_path)
                self.doc.end_paragraph()

    def summarize_notes(self):
        """
        Write a summary of all the notes in the database.
        """
        self.doc.start_paragraph("SR-Heading")
        self.doc.write_text(self._("Note Information"))
        self.doc.end_paragraph()

        self.doc.start_paragraph("SR-Normal")
        self.doc.write_text(
            self._("Number of notes: %d") % self.__db.get_number_of_notes()
        )
        self.doc.end_paragraph()


# ------------------------------------------------------------------------
#
# SummaryOptions
#
# ------------------------------------------------------------------------
class SummaryOptions(MenuReportOptions):
    """
    SummaryOptions provides the options for the SummaryReport.
    """

    def __init__(self, name, dbase):
        MenuReportOptions.__init__(self, name, dbase)

    def get_subject(self):
        """Return a string that describes the subject of the report."""
        return _("Entire Database")

    def add_menu_options(self, menu):
        """
        Add options to the menu for the summary report.
        """
        category_name = _("Report Options")

        stdoptions.add_private_data_option(menu, category_name)
        include_private_data = menu.get_option_by_name("incl_private")
        include_private_data.set_help(_("Whether to count private data"))

        stdoptions.add_living_people_option(menu, category_name, process_names=False)

        stdoptions.add_localization_option(menu, category_name)

    def make_default_style(self, default_style):
        """Make the default output style for the Summary Report."""
        font = FontStyle()
        font.set_size(16)
        font.set_type_face(FONT_SANS_SERIF)
        font.set_bold(1)
        para = ParagraphStyle()
        para.set_header_level(1)
        para.set_bottom_border(1)
        para.set_top_margin(utils.pt2cm(3))
        para.set_bottom_margin(utils.pt2cm(3))
        para.set_font(font)
        para.set_alignment(PARA_ALIGN_CENTER)
        para.set_description(_("The style used for the title."))
        default_style.add_paragraph_style("SR-Title", para)

        font = FontStyle()
        font.set_size(12)
        font.set_bold(True)
        para = ParagraphStyle()
        para.set_font(font)
        para.set_top_margin(0)
        para.set_description(_("The style used for second level headings."))
        default_style.add_paragraph_style("SR-Heading", para)

        font = FontStyle()
        font.set_size(12)
        para = ParagraphStyle()
        para.set(first_indent=-0.75, lmargin=0.75)
        para.set_font(font)
        para.set_top_margin(utils.pt2cm(3))
        para.set_bottom_margin(utils.pt2cm(3))
        para.set_description(_("The basic style used for the text display."))
        default_style.add_paragraph_style("SR-Normal", para)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# gtk
#
# -------------------------------------------------------------------------
from gi.repository import Gtk

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext

from ...widgets import MonitoredMenu, DateEntry, BasicEntry
from gramps.gen.lib import Citation
from .. import build_filter_model
from . import SidebarFilter
from gramps.gen.filters import GenericFilterFactory, rules
from gramps.gen.filters.rules.citation import (
    RegExpIdOf,
    HasCitation,
    HasTag,
    HasNoteRegexp,
    MatchesFilter,
    HasSource,
    RegExpSourceIdOf,
    HasSourceNoteRegexp,
)
from gramps.gen.utils.string import conf_strings

GenericCitationFilter = GenericFilterFactory("Citation")


# -------------------------------------------------------------------------
#
# CitationSidebarFilter class
#
# -------------------------------------------------------------------------
class CitationSidebarFilter(SidebarFilter):
    def __init__(self, dbstate, uistate, clicked):
        self.clicked_func = clicked
        self.filter_src_id = BasicEntry()
        self.filter_src_title = BasicEntry()
        self.filter_src_author = BasicEntry()
        self.filter_src_abbr = BasicEntry()
        self.filter_src_pub = BasicEntry()
        self.filter_src_note = BasicEntry()
        self.filter_id = Gtk.Entry()
        self.filter_page = Gtk.Entry()
        self.filter_date = DateEntry(uistate, [])

        self.filter_conf = Gtk.ComboBox()
        model = Gtk.ListStore(str)
        for conf_value in sorted(conf_strings.keys()):
            model.append((_(conf_strings[conf_value]),))
        self.filter_conf.set_model(model)
        self.filter_conf.set_active(Citation.CONF_VERY_LOW)

        self.filter_note = Gtk.Entry()

        self.filter_regex = Gtk.CheckButton(label=_("Use regular expressions"))
        self.sensitive_regex = Gtk.CheckButton(label=_("Case sensitive"))

        self.tag = Gtk.ComboBox()
        self.generic = Gtk.ComboBox()

        SidebarFilter.__init__(self, dbstate, uistate, "Citation")

    def create_widget(self):
        cell = Gtk.CellRendererText()
        cell.set_property("width", self._FILTER_WIDTH)
        cell.set_property("ellipsize", self._FILTER_ELLIPSIZE)
        self.generic.pack_start(cell, True)
        self.generic.add_attribute(cell, "text", 0)
        self.on_filters_changed("Citation")

        cell = Gtk.CellRendererText()
        cell.set_property("width", self._FILTER_WIDTH)
        cell.set_property("ellipsize", self._FILTER_ELLIPSIZE)
        self.filter_conf.pack_start(cell, True)
        self.filter_conf.add_attribute(cell, "text", 0)

        cell = Gtk.CellRendererText()
        cell.set_property("width", self._FILTER_WIDTH)
        cell.set_property("ellipsize", self._FILTER_ELLIPSIZE)
        self.tag.pack_start(cell, True)
        self.tag.add_attribute(cell, "text", 0)

        self.add_heading(_("Source:"))
        self.add_text_entry(_("ID"), self.filter_src_id)
        self.add_text_entry(_("Title"), self.filter_src_title)
        self.add_text_entry(_("Author"), self.filter_src_author)
        self.add_text_entry(_("Abbreviation"), self.filter_src_abbr)
        self.add_text_entry(_("Publication"), self.filter_src_pub)
        self.add_text_entry(_("Note"), self.filter_src_note)
        self.add_heading(_("Citation:"))
        self.add_text_entry(_("ID"), self.filter_id)
        self.add_text_entry(_("Volume/Page"), self.filter_page)
        self.add_text_entry(_("Date"), self.filter_date)
        self.add_entry(
            _("Min. Conf.", "Citation: Minimum Confidence"), self.filter_conf
        )
        self.add_text_entry(_("Note"), self.filter_note)
        self.add_entry(_("Tag"), self.tag)
        self.add_filter_entry(_("Custom filter"), self.generic)
        self.add_entry(None, self.filter_regex)
        self.add_entry(None, self.sensitive_regex)
        self.filter_regex.connect("toggled", self.regex_selection)
        self.regex_selection()

    def clear(self, obj):
        self.filter_src_id.set_text("")
        self.filter_src_title.set_text("")
        self.filter_src_author.set_text("")
        self.filter_src_abbr.set_text("")
        self.filter_src_pub.set_text("")
        self.filter_src_note.set_text("")
        self.filter_id.set_text("")
        self.filter_page.set_text("")
        self.filter_date.set_text("")
        self.filter_conf.set_active(Citation.CONF_NORMAL)
        self.filter_note.set_text("")
        self.tag.set_active(0)
        self.generic.set_active(0)

    def get_filter(self):
        src_id = str(self.filter_src_id.get_text()).strip()
        src_title = str(self.filter_src_title.get_text()).strip()
        src_author = str(self.filter_src_author.get_text()).strip()
        src_abbr = str(self.filter_src_abbr.get_text()).strip()
        src_pub = str(self.filter_src_pub.get_text()).strip()
        src_note = str(self.filter_src_note.get_text()).strip()
        gid = str(self.filter_id.get_text()).strip()
        page = str(self.filter_page.get_text()).strip()
        date = str(self.filter_date.get_text()).strip()
        conf = str(self.filter_conf.get_active())
        note = str(self.filter_note.get_text()).strip()
        regex = self.filter_regex.get_active()
        usecase = self.sensitive_regex.get_active()
        tag = self.tag.get_active() > 0
        gen = self.generic.get_active() > 0

        empty = not (
            src_id
            or src_title
            or src_author
            or src_abbr
            or src_pub
            or src_note
            or gid
            or page
            or date
            or conf
            or note
            or regex
            or gen
        )
        if empty:
            generic_filter = None
        else:
            generic_filter = GenericCitationFilter()
            if gid:
                rule = RegExpIdOf([gid], use_regex=regex, use_case=usecase)
                generic_filter.add_rule(rule)

            rule = HasCitation([page, date, conf], use_regex=regex, use_case=usecase)
            generic_filter.add_rule(rule)

            if src_id:
                rule = RegExpSourceIdOf([src_id], use_regex=regex, use_case=usecase)
                generic_filter.add_rule(rule)

            rule = HasSource(
                [src_title, src_author, src_abbr, src_pub],
                use_regex=regex,
                use_case=usecase,
            )
            generic_filter.add_rule(rule)

            if note:
                rule = HasNoteRegexp([note], use_regex=regex, use_case=usecase)
                generic_filter.add_rule(rule)

            if src_note:
                rule = HasSourceNoteRegexp(
                    [src_note], use_regex=regex, use_case=usecase
                )
                generic_filter.add_rule(rule)

            # check the Tag
            if tag:
                model = self.tag.get_model()
                node = self.tag.get_active_iter()
                attr = model.get_value(node, 0)
                rule = HasTag([attr])
                generic_filter.add_rule(rule)

            if self.generic.get_active() != 0:
                model = self.generic.get_model()
                node = self.generic.get_active_iter()
                obj = str(model.get_value(node, 0))
                rule = MatchesFilter([obj])
                generic_filter.add_rule(rule)

        return generic_filter

    def on_filters_changed(self, name_space):
        if name_space == "Citation":
            all_filter = GenericCitationFilter()
            all_filter.set_name(_("None"))
            all_filter.add_rule(rules.citation.AllCitations([]))
            self.generic.set_model(build_filter_model("Citation", [all_filter]))
            self.generic.set_active(0)

    def on_tags_changed(self, tag_list):
        """
        Update the list of tags in the tag filter.
        """
        model = Gtk.ListStore(str)
        model.append(("",))
        for tag_name in tag_list:
            model.append((tag_name,))
        self.tag.set_model(model)
        self.tag.set_active(0)



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2012       Nick Hall
# Copyright (C) 2012       Brian G. Matherly
# Copyright (C) 2012-2014  Paul Franklin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# ------------------------------------------------------------------------
#
# Python modules
#
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from gramps.gen.plug.report import Report
from gramps.gen.plug.report import MenuReportOptions
from gramps.gen.plug.report import stdoptions
from gramps.gen.plug.docgen import (
    FontStyle,
    ParagraphStyle,
    TableStyle,
    TableCellStyle,
    FONT_SANS_SERIF,
    IndexMark,
    INDEX_TYPE_TOC,
)


# ------------------------------------------------------------------------
#
# TableOfContents
#
# ------------------------------------------------------------------------
class TableOfContents(Report):
    """This report class generates a table of contents for a book."""

    def __init__(self, database, options, user):
        """
        Create TableOfContents object that produces the report.

        The arguments are:

        database        - the Gramps database instance
        options         - instance of the Options class for this report
        user            - a gen.user.User() instance
        """
        Report.__init__(self, database, options, user)
        self.set_locale(options.menu.get_option_by_name("trans").get_value())

    def write_report(self):
        """Generate the contents of the report"""
        mark = IndexMark(self._("Table Of Contents"), INDEX_TYPE_TOC, 1)
        self.doc.start_paragraph("TOC-Title")
        self.doc.write_text("", mark)
        self.doc.end_paragraph()
        self.doc.toc_title = self._("Contents")
        self.doc.insert_toc()


# ------------------------------------------------------------------------
#
# TableOfContentsOptions
#
# ------------------------------------------------------------------------
class TableOfContentsOptions(MenuReportOptions):
    """
    Defines options and provides handling interface.
    """

    def __init__(self, name, dbase):
        self.__db = dbase
        MenuReportOptions.__init__(self, name, dbase)

    def get_subject(self):
        """Return a string that describes the subject of the report."""
        return _("Entire Book")

    def add_menu_options(self, menu):
        """Add the options for this report"""
        category_name = _("Report Options")
        stdoptions.add_localization_option(menu, category_name)

    def make_default_style(self, default_style):
        """Make the default output style for the TableOfContents report."""
        font = FontStyle()
        font.set(face=FONT_SANS_SERIF, size=14)
        para = ParagraphStyle()
        para.set_font(font)
        para.set_bottom_margin(0.25)
        para.set_description(_("The style used for the title."))
        default_style.add_paragraph_style("TOC-Title", para)

        table = TableStyle()
        table.set_width(100)
        table.set_columns(2)
        table.set_column_width(0, 80)
        table.set_column_width(1, 20)
        default_style.add_table_style("TOC-Table", table)

        cell = TableCellStyle()
        default_style.add_cell_style("TOC-Cell", cell)

        para = ParagraphStyle()
        para.set_font(font)
        para.set_description(_("The style used for first level headings."))
        default_style.add_paragraph_style("TOC-Heading1", para)

        para = ParagraphStyle()
        para.set_font(font)
        para.set_first_indent(0.5)
        para.set_description(_("The style used for second level headings."))
        default_style.add_paragraph_style("TOC-Heading2", para)

        para = ParagraphStyle()
        para.set_font(font)
        para.set_first_indent(1)
        para.set_description(_("The style used for third level headings."))
        default_style.add_paragraph_style("TOC-Heading3", para)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2010       Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Python classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext

# -------------------------------------------------------------------------
#
# GTK classes
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import Pango

_TAB = Gdk.keyval_from_name("Tab")
_ENTER = Gdk.keyval_from_name("Enter")

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from .surnamemodel import SurnameModel
from .embeddedlist import EmbeddedList, TEXT_EDIT_COL
from ...ddtargets import DdTargets
from gramps.gen.lib import Surname, NameOriginType
from ...utils import match_primary_mask, no_match_primary_mask

# table for skipping illegal control chars
INVISIBLE = dict.fromkeys(list(range(32)))


# -------------------------------------------------------------------------
#
# SurnameTab
#
# -------------------------------------------------------------------------
class SurnameTab(EmbeddedList):
    _HANDLE_COL = 5
    _DND_TYPE = DdTargets.SURNAME

    _MSG = {
        "add": _("Create and add a new surname"),
        "del": _("Remove the selected surname"),
        "edit": _("Edit the selected surname"),
        "up": _("Move the selected surname upwards"),
        "down": _("Move the selected surname downwards"),
    }

    # index = column in model. Value =
    #  (name, sortcol in model, width, markup/text
    _column_names = [
        (_("Prefix"), 0, 150, TEXT_EDIT_COL, -1, None),
        (_("Surname"), 1, -1, TEXT_EDIT_COL, -1, None),
        (_("Connector"), 2, 100, TEXT_EDIT_COL, -1, None),
    ]
    _column_combo = (_("Origin"), -1, 150, 3)  # name, sort, width, modelcol
    _column_toggle = (_("Primary", "Name"), -1, 80, 4)

    def __init__(
        self,
        dbstate,
        uistate,
        track,
        name,
        config_key,
        on_change=None,
        top_label="<b>%s</b>" % _("Multiple Surnames"),
    ):
        self.obj = name
        self.on_change = on_change
        self.curr_col = -1
        self.curr_cellr = None
        self.curr_celle = None
        self.curr_path = None

        EmbeddedList.__init__(
            self,
            dbstate,
            uistate,
            track,
            _("Family Surnames"),
            SurnameModel,
            config_key,
            move_buttons=True,
            top_label=top_label,
        )

    def build_columns(self):
        # first the standard text columns with normal method
        EmbeddedList.build_columns(self)

        # now we add the two special columns
        # combobox for type
        colno = len(self.columns)
        name = self._column_combo[0]
        renderer = Gtk.CellRendererCombo()
        renderer.set_property("ellipsize", Pango.EllipsizeMode.END)
        # set up the comboentry editable
        no = NameOriginType()
        self.cmborig = Gtk.ListStore(GObject.TYPE_INT, GObject.TYPE_STRING)
        self.cmborigmap = no.get_map().copy()
        # sort the keys based on the value
        keys = sorted(
            self.cmborigmap, key=lambda x: glocale.sort_key(self.cmborigmap[x])
        )
        for key in keys:
            if key != no.get_custom():
                self.cmborig.append(row=[key, self.cmborigmap[key]])
        additional = self.dbstate.db.get_origin_types()
        if additional:
            for type in additional:
                if type:
                    self.cmborig.append(row=[no.get_custom(), type])
        renderer.set_property("model", self.cmborig)
        renderer.set_property("text-column", 1)
        renderer.set_property("editable", not self.dbstate.db.readonly)

        renderer.connect("editing_started", self.on_edit_start_cmb, colno)
        renderer.connect("edited", self.on_orig_edited, self._column_combo[3])
        # add to treeview
        column = Gtk.TreeViewColumn(name, renderer, text=self._column_combo[3])
        column.set_resizable(True)
        column.set_sort_column_id(self._column_combo[1])
        column.set_min_width(self._column_combo[2])
        column.set_expand(False)
        self.columns.append(column)
        self.tree.append_column(column)
        # toggle box for primary
        colno += 1
        name = self._column_toggle[0]
        renderer = Gtk.CellRendererToggle()
        renderer.set_property("activatable", True)
        renderer.set_property("radio", True)
        renderer.connect("toggled", self.on_prim_toggled, self._column_toggle[3])
        # add to treeview
        column = Gtk.TreeViewColumn(name, renderer, active=self._column_toggle[3])
        column.set_resizable(False)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        column.set_alignment(0.5)
        column.set_sort_column_id(self._column_toggle[1])
        column.set_max_width(self._column_toggle[2])
        self.columns.append(column)
        self.tree.append_column(column)

    def setup_editable_col(self):
        """
        inherit this and set the variables needed for editable columns
        Variable edit_col_funcs needs to be a dictionary from model col_nr to
        function to call for
        Example:
        self.edit_col_funcs ={1: {'edit_start': self.on_edit_start,
                                  'edited': self.on_edited
                              }}
        """
        self.edit_col_funcs = {
            0: {"edit_start": self.on_edit_start, "edited": self.on_edit_inline},
            1: {"edit_start": self.on_edit_start, "edited": self.on_edit_inline},
            2: {"edit_start": self.on_edit_start, "edited": self.on_edit_inline},
        }

    def get_data(self):
        return self.obj.get_surname_list()

    def is_empty(self):
        return len(self.model) == 0

    def _get_surn_from_model(self):
        """
        Return new surname_list for storing in the name based on content of
        the model
        """
        new_list = []
        for idx, value in enumerate(self.model):
            node = self.model.get_iter(idx)
            surn = self.model.get_value(node, 5)
            surn.set_prefix(self.model.get_value(node, 0))
            surn.set_surname(self.model.get_value(node, 1))
            surn.set_connector(self.model.get_value(node, 2))
            surn.get_origintype().set(self.model.get_value(node, 3))
            surn.set_primary(self.model.get_value(node, 4))
            new_list += [surn]
        return new_list

    def update(self):
        """
        Store the present data in the model to the name object
        """
        new_map = self._get_surn_from_model()
        self.obj.set_surname_list(new_map)
        # update name in previews
        if self.on_change:
            self.on_change()

    def post_rebuild(self, prebuildpath):
        """
        Called when data model has changed, in particular necessary when row
        order is updated.
        @param prebuildpath: path selected before rebuild, None if none
        @type prebuildpath: tree path
        """
        if self.on_change:
            self.on_change()

    def column_order(self):
        # order of columns for EmbeddedList. Only the text columns here
        return ((1, 0), (1, 1), (1, 2))

    def add_button_clicked(self, obj):
        """Add button is clicked, add a surname to the person"""
        prim = False
        if len(self.obj.get_surname_list()) == 0:
            prim = True
        node = self.model.append(
            row=["", "", "", str(NameOriginType()), prim, Surname()]
        )
        self.selection.select_iter(node)
        path = self.model.get_path(node)
        self.tree.set_cursor_on_cell(
            path, focus_column=self.columns[0], focus_cell=None, start_editing=True
        )
        self.update()

    def del_button_clicked(self, obj):
        """
        Delete button is clicked. Remove from the model
        """
        (_model, node) = self.selection.get_selected()
        if node:
            path = self.model.get_path(node).to_string()
            if path == self.curr_path:
                self.curr_path = None
            self.model.remove(node)
            self.update()

    def on_edit_start(self, cellr, celle, path, colnr):
        """start of editing. Store stuff so we know when editing ends where we
        are
        """
        self.curr_col = colnr
        self.curr_cellr = cellr
        self.curr_celle = celle
        self.curr_path = path

    def on_edit_start_cmb(self, cellr, celle, path, colnr):
        """
        An edit starts in the origin type column
        This means a cmb has been created as celle, and we can set up the stuff
        we want this cmb to contain: autocompletion, stop edit when selection
        in the cmb happens.
        """
        self.on_edit_start(cellr, celle, path, colnr)
        # set up autocomplete
        entry = celle.get_child()
        entry.set_width_chars(10)
        completion = Gtk.EntryCompletion()
        completion.set_model(self.cmborig)
        completion.set_minimum_key_length(1)
        completion.set_text_column(1)
        entry.set_completion(completion)
        #
        celle.connect("changed", self.on_origcmb_change, path, colnr)

    def on_edit_start_toggle(self, cellr, celle, path, colnr):
        """
        Edit
        """
        self.on_edit_start(cellr, celle, path, colnr)

    def on_edit_inline(self, cell, path, new_text, colnr):
        """
        Edit is happening. The model is updated and the surname objects updated.
        colnr must be the column in the model.
        """
        if self.curr_path == None:
            return
        node = self.model.get_iter(path)
        text = new_text.translate(INVISIBLE).strip()
        self.model.set_value(node, colnr, text)
        self.update()

    def on_orig_edited(self, cellr, path, new_text, colnr):
        """
        An edit is finished in the origin type column. For a cmb in an editor,
        the model may only be updated when typing is finished, as editing stops
        automatically on update of the model.
        colnr must be the column in the model.
        """
        self.on_edit_inline(cellr, path, new_text, colnr)

    def on_origcmb_change(self, cmb, path, colnr):
        """
        A selection occured in the cmb of the origin type column. colnr must
        be the column in the model.
        """
        act = cmb.get_active()
        if act == -1:
            return
        self.on_orig_edited(
            None, path, self.cmborig.get_value(self.cmborig.get_iter((act,)), 1), colnr
        )

    def on_prim_toggled(self, cell, path, colnr):
        """
        Primary surname on path is toggled. colnr must be the col
        in the model
        """
        # obtain current value
        node = self.model.get_iter(path)
        old_val = self.model.get_value(node, colnr)
        for nr, surname in enumerate(self.obj.get_surname_list()):
            if nr == int(path[0]):
                if old_val:
                    # True remains True
                    break
                else:
                    # This value becomes True
                    self.model.set_value(self.model.get_iter((nr,)), colnr, True)
            else:
                self.model.set_value(self.model.get_iter((nr,)), colnr, False)
        self.update()
        return

    def edit_button_clicked(self, obj):
        """Edit button clicked"""
        (model, node) = self.selection.get_selected()
        if node:
            path = self.model.get_path(node)
            self.tree.set_cursor_on_cell(
                path, focus_column=self.columns[0], focus_cell=None, start_editing=True
            )

    def key_pressed(self, obj, event):
        """
        Handles the key being pressed.
        Here we make sure tab moves to next or previous value in row on TAB
        """
        if not EmbeddedList.key_pressed(self, obj, event):
            if event.type == Gdk.EventType.KEY_PRESS and event.keyval in (_TAB,):
                if no_match_primary_mask(
                    event.get_state(), Gdk.ModifierType.SHIFT_MASK
                ):
                    return self.next_cell()
                elif match_primary_mask(event.get_state(), Gdk.ModifierType.SHIFT_MASK):
                    return self.prev_cell()
                else:
                    return
            else:
                return
        return True

    def next_cell(self):
        """
        Move to the next cell to edit it
        """
        (model, node) = self.selection.get_selected()
        if node:
            path = self.model.get_path(node).get_indices()[0]
            nccol = self.curr_col + 1
            if nccol < 4:
                if self.curr_celle:
                    self.curr_celle.editing_done()
                self.tree.set_cursor_on_cell(
                    Gtk.TreePath((path,)),
                    focus_column=self.columns[nccol],
                    focus_cell=None,
                    start_editing=True,
                )
            elif nccol == 4:
                # go to next line if there is one
                if path < len(self.obj.get_surname_list()):
                    newpath = Gtk.TreePath((path + 1,))
                    self.curr_celle.editing_done()
                    self.selection.select_path(newpath)
                    self.tree.set_cursor_on_cell(
                        newpath,
                        focus_column=self.columns[0],
                        focus_cell=None,
                        start_editing=True,
                    )
                else:
                    # stop editing
                    self.curr_celle.editing_done()
                    return
        return True

    def prev_cell(self):
        """
        Move to the next cell to edit it
        """
        (model, node) = self.selection.get_selected()
        if node:
            path = self.model.get_path(node).get_indices()[0]
            if self.curr_col > 0:
                self.tree.set_cursor_on_cell(
                    Gtk.TreePath((path,)),
                    focus_column=self.columns[self.curr_col - 1],
                    focus_cell=None,
                    start_editing=True,
                )
            elif self.curr_col == 0:
                # go to prev line if there is one
                if path > 0:
                    newpath = Gtk.TreePath((path - 1,))
                    self.selection.select_path(newpath)
                    self.tree.set_cursor_on_cell(
                        newpath,
                        focus_column=self.columns[-2],
                        focus_cell=None,
                        start_editing=True,
                    )
                else:
                    # stop editing
                    self.curr_celle.editing_done()
                    return
        return True



#!/usr/bin/env python3
import os
from os import environ
from os.path import join
import sys
import site

if getattr(sys, "frozen", False):
    aio = os.path.dirname(sys.executable)
    sys.path.insert(1, aio)
    sys.path.insert(1, os.path.join(aio, "lib"))
    sys.path.insert(1, site.getusersitepackages())
    environ["PANGOCAIRO_BACKEND"] = "fontconfig"
    environ["SSL_CERT_FILE"] = join(aio, "ssl/certs/ca-bundle.trust.crt")
    environ["GI_TYPELIB_PATH"] = join(aio, "lib/girepository-1.0")
    environ["G_ENABLE_DIAGNOSTIC"] = "0"
    environ["G_PARAM_DEPRECATED"] = "0"
    environ["GRAMPS_RESOURCES"] = join(aio, "share")
    environ["PATH"] = aio + ";" + aio + "\\lib;" + environ["PATH"]


def close():
    sys.exit()


import atexit
import ctypes

HANDLE = ctypes.windll.kernel32.CreateMutexW(None, 1, "org.gramps-project.gramps")
ERROR = ctypes.GetLastError()
if ERROR == 183:  # ERROR_ALREADY_EXISTS:
    print("Gramps is already running!", file=sys.stderr)
    close()

atexit.register(ctypes.windll.kernel32.CloseHandle, HANDLE)
atexit.register(ctypes.windll.kernel32.ReleaseMutex, HANDLE)

import warnings

warnings.simplefilter("ignore")

import gramps.grampsapp as app

app.run()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2009 Benny Malengier
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
from gramps.gen.plug._pluginreg import *
from gramps.gen.const import URL_MANUAL_PAGE, GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

MODULE_VERSION = "6.0"
QUICKVIEWS_HELP = URL_MANUAL_PAGE + "_-_Reports#Quick_Views"

# ------------------------------------------------------------------------
#
# Age on Date
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="ageondate",
    name=_("Age on Date"),
    description=_("Display people and ages on a particular date"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="ageondate.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_DATE,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Attribute Match
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="attribute_match",
    name=_("Attribute Match"),
    description=_("Display people with same attribute."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="attributematch.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_MISC,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# All Events
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="all_events",
    name=_("All Events"),
    description=_("Display a person's events, both personal and family."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="all_events.py",
    authors=["Donald N. Allingham"],
    authors_email=["don@gramps-project.org"],
    category=CATEGORY_QR_PERSON,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)


register(
    QUICKREPORT,
    id="all_events_fam",
    name=_("All Family Events"),
    description=_("Display the family and family members events."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="all_events.py",
    authors=["B. Malengier"],
    authors_email=["benny.malengier@gramps-project.org"],
    category=CATEGORY_QR_FAMILY,
    runfunc="run_fam",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Relation to Home Person
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="all_relations",
    name=_("Relation to Home Person"),
    description=_("Display all relationships between person and home person."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="all_relations.py",
    authors=["B. Malengier"],
    authors_email=["benny.malengier@gramps-project.org"],
    category=CATEGORY_QR_PERSON,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Filter
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="filterbyname",
    name=_("Filter"),
    description=_("Display filtered data"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="filterbyname.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_MISC,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Father/mother lineage
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="father_lineage",
    name=_("Father lineage"),
    description=_("Display father lineage"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="lineage.py",
    authors=["B. Malengier"],
    authors_email=["benny.malengier@gramps-project.org"],
    category=CATEGORY_QR_PERSON,
    runfunc="run_father",
    help_url=QUICKVIEWS_HELP,
)

register(
    QUICKREPORT,
    id="mother_lineage",
    name=_("Mother lineage"),
    description=_("Display mother lineage"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="lineage.py",
    authors=["B. Malengier"],
    authors_email=["benny.malengier@gramps-project.org"],
    category=CATEGORY_QR_PERSON,
    runfunc="run_mother",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# On This Day
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="onthisday",
    name=_("On This Day"),
    description=_("Display events on a particular day"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="onthisday.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_EVENT,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# References
#
# ------------------------------------------------------------------------

refitems = [
    (CATEGORY_QR_PERSON, "person", _("Person")),
    (CATEGORY_QR_FAMILY, "family", _("Family")),
    (CATEGORY_QR_EVENT, "event", _("Event")),
    (CATEGORY_QR_SOURCE, "source", _("Source")),
    (CATEGORY_QR_PLACE, "place", _("Place")),
    (CATEGORY_QR_MEDIA, "media", _("Media")),
    (CATEGORY_QR_NOTE, "note", _("Note")),
    (CATEGORY_QR_CITATION, "citation", _("Citation")),
    (CATEGORY_QR_SOURCE_OR_CITATION, "source_or_citation", _("Source or Citation")),
]

for category, item, trans in refitems:
    register(
        QUICKREPORT,
        id=item + "references",
        name=_("%s References") % trans,
        description=_("Display references for a %s") % trans,
        version="1.0",
        gramps_target_version=MODULE_VERSION,
        status=STABLE,
        fname="references.py",
        authors=["Douglas Blank"],
        authors_email=["doug.blank@gmail.com"],
        category=category,
        runfunc="run_%s" % item,
        help_url=QUICKVIEWS_HELP,
    )

register(
    QUICKREPORT,
    id="link_references",
    name=_("Link References"),
    description=_("Display link references for a note"),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="linkreferences.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_NOTE,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Show Repository Reference
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="RepoRef",
    name=_("Repository References"),
    description=_(
        "Display the repository reference for sources related to"
        " the active repository"
    ),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="reporef.py",
    authors=["Jerome Rapinat"],
    authors_email=["romjerome@yahoo.fr"],
    category=CATEGORY_QR_REPOSITORY,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# Same Surnames/Given names
#
# ------------------------------------------------------------------------

register(
    QUICKREPORT,
    id="samesurnames",
    name=_("Same Surnames"),
    description=_("Display people with the same surname as a person."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="samesurnames.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_PERSON,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)

register(
    QUICKREPORT,
    id="samegivens",
    name=_("Same Given Names"),
    description=_("Display people with the same given name as a person."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="samesurnames.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_PERSON,
    runfunc="run_given",
    help_url=QUICKVIEWS_HELP,
)

register(
    QUICKREPORT,
    id="samegivens_misc",
    name=_("Same Given Names - stand-alone"),
    description=_("Display people with the same given name as a person."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="samesurnames.py",
    authors=["Douglas Blank"],
    authors_email=["doug.blank@gmail.com"],
    category=CATEGORY_QR_MISC,
    runfunc="run_given",
    help_url=QUICKVIEWS_HELP,
)

# ------------------------------------------------------------------------
#
# siblings
#
# ------------------------------------------------------------------------
register(
    QUICKREPORT,
    id="siblings",
    name=_("Siblings"),
    description=_("Display a person's siblings."),
    version="1.0",
    gramps_target_version=MODULE_VERSION,
    status=STABLE,
    fname="siblings.py",
    authors=["Donald N. Allingham"],
    authors_email=["don@gramps-project.org"],
    category=CATEGORY_QR_PERSON,
    runfunc="run",
    help_url=QUICKVIEWS_HELP,
)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Danish-specific classes for parsing and displaying dates.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import re

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler


# -------------------------------------------------------------------------
#
# Danish parser class
#
# -------------------------------------------------------------------------
class DateParserDa(DateParser):
    """
    Convert a text string into a Date object, expecting a date
    notation in the Danish language. If the date cannot be converted,
    the text string is assigned.
    """

    # modifiers before the date
    modifier_to_int = {
        "før": Date.MOD_BEFORE,
        "inden": Date.MOD_BEFORE,
        "efter": Date.MOD_AFTER,
        "omkring": Date.MOD_ABOUT,
        "ca.": Date.MOD_ABOUT,
        "fra": Date.MOD_FROM,
        "til": Date.MOD_TO,
    }

    bce = ["f Kr"]

    calendar_to_int = {
        "gregoriansk   ": Date.CAL_GREGORIAN,
        "g": Date.CAL_GREGORIAN,
        "juliansk tidsregning": Date.CAL_JULIAN,
        "juliansk": Date.CAL_JULIAN,
        "j": Date.CAL_JULIAN,
        "hebraisk": Date.CAL_HEBREW,
        "h": Date.CAL_HEBREW,
        "islamisk": Date.CAL_ISLAMIC,
        "muslimsk": Date.CAL_ISLAMIC,
        "i": Date.CAL_ISLAMIC,
        "fransk": Date.CAL_FRENCH,
        "fransk republikansk": Date.CAL_FRENCH,
        "f": Date.CAL_FRENCH,
        "persisk": Date.CAL_PERSIAN,
        "p": Date.CAL_PERSIAN,
        "svensk": Date.CAL_SWEDISH,
        "s": Date.CAL_SWEDISH,
    }

    quality_to_int = {
        "estimeret": Date.QUAL_ESTIMATED,
        "anslået": Date.QUAL_ESTIMATED,
        "beregnet": Date.QUAL_CALCULATED,
    }

    def init_strings(self):
        DateParser.init_strings(self)
        self._span = re.compile(
            r"(fra)?\s*(?P<start>.+)\s*(til|--|–)\s*(?P<stop>.+)", re.IGNORECASE
        )
        self._range = re.compile(
            r"(mellem)\s+(?P<start>.+)\s+og\s+(?P<stop>.+)", re.IGNORECASE
        )


# -------------------------------------------------------------------------
#
# Danish display class
#
# -------------------------------------------------------------------------
class DateDisplayDa(DateDisplay):
    """
    Danish language date display class.
    """

    long_months = (
        "",
        "januar",
        "februar",
        "marts",
        "april",
        "maj",
        "juni",
        "juli",
        "august",
        "september",
        "oktober",
        "november",
        "december",
    )

    short_months = (
        "",
        "jan",
        "feb",
        "mar",
        "apr",
        "maj",
        "jun",
        "jul",
        "aug",
        "sep",
        "okt",
        "nov",
        "dec",
    )

    formats = (
        "ÅÅÅÅ-MM-DD (ISO)",
        "Numerisk",
        "Måned dag, år",
        "Md Dag År",
        "Dag måned år",
        "Dag md År",
    )
    # this must agree with DateDisplayEn's "formats" definition
    # (since no locale-specific _display_gregorian exists, here)

    calendar = (
        "",
        "juliansk",
        "hebraisk",
        "fransk republikansk",
        "persisk",
        "islamisk",
        "svensk",
    )

    _mod_str = ("", "før ", "efter ", "ca. ", "", "", "")

    _qual_str = ("", "beregnet ", "beregnet ")

    _bce_str = "%s f. Kr."

    def display(self, date):
        """
        Return a text string representing the date.
        """
        mod = date.get_modifier()
        cal = date.get_calendar()
        qual = date.get_quality()
        start = date.get_start_date()
        newyear = date.get_new_year()

        qual_str = self._qual_str[qual]

        if mod == Date.MOD_TEXTONLY:
            return date.get_text()
        elif start == Date.EMPTY:
            return ""
        elif mod == Date.MOD_SPAN:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%sfra %s til %s%s" % (qual_str, d1, d2, scal)
        elif mod == Date.MOD_RANGE:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%smellem %s og %s%s" % (qual_str, d1, d2, scal)
        else:
            text = self.display_cal[date.get_calendar()](start)
            scal = self.format_extras(cal, newyear)
            return "%s%s%s%s" % (qual_str, self._mod_str[mod], text, scal)


# -------------------------------------------------------------------------
#
# Register classes
#
# -------------------------------------------------------------------------
register_datehandler(
    ("da_DK", "da", "dansk", "Danish", ("%d-%m-%Y",)), DateParserDa, DateDisplayDa
)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2010  Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
SurnameBase class for Gramps.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..const import GRAMPS_LOCALE as glocale
from .const import EQUAL, IDENTICAL
from .surname import Surname

_ = glocale.translation.gettext


# -------------------------------------------------------------------------
#
# SurnameBase
#
# -------------------------------------------------------------------------
class SurnameBase:
    """
    Base class for surname-aware objects.
    """

    def __init__(self, source=None):
        """
        Initialize a SurnameBase.

        If the source is not None, then object is initialized from values of
        the source object.

        :param source: Object used to initialize the new object
        :type source: SurnameBase
        """
        self.surname_list = list(map(Surname, source.surname_list)) if source else []

    def serialize(self):
        """
        Convert the object to a serialized tuple of data.
        """
        return [surname.serialize() for surname in self.surname_list]

    def unserialize(self, data):
        """
        Convert a serialized tuple of data to an object.
        """
        self.surname_list = [Surname().unserialize(item) for item in data]

    def add_surname(self, surname):
        """
        Add the :class:`~.surname.Surname` instance to the object's
        list of surnames.

        :param surname: :class:`~.surname.Surname` instance to add to the
                        object's address list.
        :type address: list
        """
        self.surname_list.append(surname)

    def remove_surname(self, surname):
        """
        Remove the specified :class:`~.surname.Surname` instance from the
        surname list.

        If the instance does not exist in the list, the operation has
        no effect.

        :param surname: :class:`~.surname.Surname` instance to remove
                        from the list
        :type surname: :class:`~.surname.Surname`

        :returns: True if the surname was removed, False if it was not in the
                  list.
        :rtype: bool
        """
        if surname in self.surname_list:
            self.surname_list.remove(surname)
            return True
        return False

    def get_surname_list(self):
        """
        Return the list of :class:`~.surname.Surname` instances
        associated with the object.

        :returns: Returns the list of :class:`~.surname.Surname` instances
        :rtype: list
        """
        return self.surname_list

    def set_surname_list(self, surname_list):
        """
        Assign the passed list to the object's list of
        :class:`~.surname.Surname` instances.

        :param surname_list: List of :class:`~.surname.Surname` instances to be
                             associated with the object
        :type surname_list: list
        """
        self.surname_list = surname_list

    def get_primary_surname(self):
        """
        Return the surname that is the primary surname

        :returns: Returns the surname instance that is the primary surname. If
                  primary not set, and there is a surname, the first surname is
                  given, if no surnames, None is returned
        :rtype: :class:`~.surname.Surname` or None
        """
        for surname in self.surname_list:
            if surname.primary:
                return surname
        if self.surname_list:
            return self.surname_list[0]
        # self healing, add a surname to this object and return it
        self.set_surname_list([Surname()])
        return self.surname_list[0]

    def set_primary_surname(self, surnamenr=0):
        """
        Set the surname with surnamenr in the surname list as primary surname
        Counting starts at 0
        """
        assert isinstance(
            surnamenr, int
        ), "Surname.set_primary_surname requires integer"
        if surnamenr >= len(self.surname_list):
            return
        for surname in self.surname_list:
            surname.set_primary(False)
        self.surname_list[surnamenr].set_primary(True)

    def _merge_surname_list(self, acquisition):
        """
        Merge the list of surname from acquisition with our own.
        This method is normally only called when surnames are equal, if they
        are different, the merge code should fall back to storing an
        alternate name. For completeness, the code is present nevertheless.

        :param acquisition: the surname list of this object will be merged with
                            the current surname list.
        :type acquisition: SurnameBase
        """
        surname_list = self.surname_list[:]
        for addendum in acquisition.get_surname_list():
            for surname in surname_list:
                equi = surname.is_equivalent(addendum)
                if equi == IDENTICAL:
                    break
                if equi == EQUAL:
                    # This should normally never happen, an alternate name
                    # should be added
                    surname.merge(addendum)
                    break
            else:
                self.surname_list.append(addendum)

    def get_surname(self):
        """
        Return a fully formatted surname utilizing the surname_list
        """
        totalsurn = ""
        for surn in self.surname_list:
            partsurn = surn.get_surname()
            if surn.get_prefix():
                fsurn = _("%(first)s %(second)s") % {
                    "first": surn.get_prefix(),
                    "second": partsurn,
                }
            else:
                fsurn = partsurn
            fsurn = fsurn.strip()
            if surn.get_connector():
                fsurn = _("%(first)s %(second)s") % {
                    "first": fsurn,
                    "second": surn.get_connector(),
                }
            fsurn = fsurn.strip()
            totalsurn = _("%(first)s %(second)s") % {
                "first": totalsurn,
                "second": fsurn,
            }
        return totalsurn.strip()

    def get_primary(self):
        """
        Return a fully formatted primary surname
        """
        primary = self.get_primary_surname()
        partsurn = primary.get_surname()
        if primary.get_prefix():
            fsurn = _("%(first)s %(second)s") % {
                "first": primary.get_prefix(),
                "second": partsurn,
            }
        else:
            fsurn = partsurn
        return fsurn.strip()

    def get_upper_surname(self):
        """Return a fully formatted surname capitalized"""
        return self.get_surname().upper()

    def get_surnames(self):
        """
        Return a list of surnames (no prefix or connectors)
        """
        surnl = []
        for surn in self.surname_list:
            realsurn = surn.get_surname()
            if realsurn:
                surnl.append(realsurn)
        return surnl

    def get_prefixes(self):
        """
        Return a list of prefixes
        """
        prefixl = []
        for surn in self.surname_list:
            prefix = surn.get_prefix()
            if prefix:
                prefixl.append(prefix)
        return prefixl

    def get_connectors(self):
        """
        Return a list of surnames (no prefix or connectors)
        """
        connl = []
        for surn in self.surname_list:
            conn = surn.get_connector()
            if conn:
                connl.append(conn)
        return connl



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Python classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
from gi.repository import Gtk
from gi.repository import GObject
from gi.repository import GLib

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from gramps.gen.lib import Url
from gramps.gen.errors import WindowActiveError
from ...ddtargets import DdTargets
from .webmodel import WebModel
from .embeddedlist import EmbeddedList, TEXT_COL, MARKUP_COL, ICON_COL


# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------
class WebEmbedList(EmbeddedList):
    _HANDLE_COL = 4
    _DND_TYPE = DdTargets.URL

    _MSG = {
        "add": _("Create and add a new web address"),
        "del": _("Remove the existing web address"),
        "edit": _("Edit the selected web address"),
        "up": _("Move the selected web address upwards"),
        "down": _("Move the selected web address downwards"),
        "jump": _("Jump to the selected web address"),
    }

    # index = column in model. Value =
    #  (name, sortcol in model, width, markup/text, weigth_col
    _column_names = [
        (_("Type"), 0, 100, TEXT_COL, -1, None),
        (_("Path"), 1, 200, TEXT_COL, -1, None),
        (_("Description"), 2, 150, TEXT_COL, -1, None),
        (_("Private"), 3, 30, ICON_COL, -1, "gramps-lock"),
    ]

    def __init__(self, dbstate, uistate, track, data, config_key):
        self.data = data
        EmbeddedList.__init__(
            self,
            dbstate,
            uistate,
            track,
            _("_Internet"),
            WebModel,
            config_key,
            move_buttons=True,
            jump_button=True,
        )

    def get_icon_name(self):
        return "gramps-url"

    def get_data(self):
        return self.data

    def column_order(self):
        return ((1, 3), (1, 0), (1, 1), (1, 2))

    def add_button_clicked(self, obj):
        from .. import EditUrl

        url = Url()
        try:
            EditUrl(self.dbstate, self.uistate, self.track, "", url, self.add_callback)
        except WindowActiveError:
            pass

    def add_callback(self, url):
        data = self.get_data()
        data.append(url)
        self.rebuild()
        GLib.idle_add(self.tree.scroll_to_cell, len(data) - 1)

    def edit_button_clicked(self, obj):
        from .. import EditUrl

        url = self.get_selected()
        if url:
            try:
                EditUrl(
                    self.dbstate, self.uistate, self.track, "", url, self.edit_callback
                )
            except WindowActiveError:
                pass

    def edit_callback(self, url):
        self.rebuild()

    def get_popup_menu_items(self):
        return [
            (True, _("_Add"), self.add_button_clicked),
            (False, _("_Edit"), self.edit_button_clicked),
            (True, _("_Remove"), self.del_button_clicked),
            (True, _("_Jump to"), self.jump_button_clicked),
        ]

    def jump_button_clicked(self, obj):
        from ...display import display_url

        url = self.get_selected()
        if url.get_path():
            display_url(url.get_path())
